<?xml version="1.0" encoding="iso-8859-1"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">

  <xsl:output method="html"/>

  <xsl:template match="/">
    <html>
      <head>
        <title>Lua-Quick-Try-Out Function Reference</title>
      </head>
      <body>
        
        <h1>Lua-Quick-Try-Out Function Reference</h1>
        
        <h2>Overview</h2>
        
        <table>
        <xsl:for-each select="MeasLabDoc/Table">
          <xsl:sort select="@Name"/>
          
          <xsl:variable name="link_idx_name" select="concat('table_idx_', @Name)" />
          <tr>
            <td>
              <b><a href="#{$link_idx_name}"><xsl:value-of select="@Name" /></a></b>
            </td>
            <td>
              <xsl:value-of select="Desc" />
            </td>
          </tr>
          
        </xsl:for-each>
        </table>
        
        <h2>LUA functions</h2>
        
        <xsl:for-each select="MeasLabDoc/Table">
          <xsl:sort select="@Name"/>
          
          <xsl:variable name="link_name" select="concat('table_', @Name)" />
          <xsl:variable name="link_idx_name" select="concat('table_idx_', @Name)" />
          <h3>Table <a href="#{$link_name}" name="{$link_idx_name}"><xsl:value-of select="@Name" /></a></h3>
          
          <xsl:if test="Func">
            <h4>functions:</h4>
          </xsl:if>
          
          <table>
            <xsl:for-each select="Func">
              <xsl:sort select="@Name"/>
              
              <xsl:variable name="full_name" >
                <xsl:if test="not(@Object)">
                  <xsl:value-of select="concat(../@Name, '.', @Name)" />
                </xsl:if>
                
                <xsl:if test="@Object">
                  <xsl:value-of select="concat(@Object, ':', @Name)" />
                </xsl:if>
              </xsl:variable>
              
              <tr>
                <td>
                  <a href="#{$full_name}"><xsl:value-of select="$full_name" /></a>
                </td>
                <td>
                  <xsl:value-of select="Desc" />
                </td>
              </tr>
              
            </xsl:for-each>
          </table>
          
          <xsl:if test="Prop">
            <h4>properties:</h4>
          </xsl:if>
          
          <table>
          
            <xsl:for-each select="Prop">
              <xsl:sort select="@Name"/>
              <xsl:variable name="full_name" select="concat(../@Name, '.', @Name)" />
              
              <tr>
                <td>
                  <a href="#{$full_name}"><xsl:value-of select="$full_name" /></a>
                </td>
                <td>
                  <xsl:value-of select="Desc" />
                </td>
              </tr>
            </xsl:for-each>
            
          </table>
          
        </xsl:for-each>
        
        <h2>LUA functions</h2>

        <xsl:apply-templates select="MeasLabDoc/Table"> 
          <xsl:sort select="@Name"/>
        </xsl:apply-templates>
        
      </body>
    </html>
  </xsl:template>

  <xsl:template match="Param">
    <tr>
      <td><b><xsl:value-of select="@Name" /></b></td>
      <td>
        <xsl:value-of select="@Desc" />
        <xsl:if test="@Opt = 1">
          (This parameter is optional.)
        </xsl:if>
      </td>
    </tr>
  </xsl:template>

  <xsl:template match="Return">
    <tr>
      <td><b><xsl:value-of select="@Name" /></b></td>
      <td><xsl:value-of select="@Desc" /></td>
    </tr>
  </xsl:template>

  <xsl:template match="Table">
    <xsl:variable name="link_name" select="concat('table_', @Name)" />
    <hr />
    <h3><a name="{$link_name}">Table: <xsl:value-of select="@Name" /></a></h3>
    <p><b><xsl:value-of select="Desc" /></b></p>
    <hr />
    <xsl:apply-templates select="Func|Prop"> 
      <xsl:sort select="@Name"/>
    </xsl:apply-templates>
  </xsl:template>

  <xsl:template match="Func">
    
    <xsl:variable name="full_name" >
      <xsl:if test="not(@Object)">
        <xsl:value-of select="concat(../@Name, '.', @Name)" />
      </xsl:if>
      
      <xsl:if test="@Object">
        <xsl:value-of select="concat(@Object, ':', @Name)" />
      </xsl:if>
    </xsl:variable>
    
    <p>
      <table border="1" cellpadding="5" cellspacing="0" width="100%">
        <tr>
          <td colspan="2"><big><b><a name="{$full_name}"><xsl:value-of select="$full_name" /></a></b> (function)</big></td>
        </tr>
        <tr>
          <td colspan="2">
            <xsl:for-each select="Return">
              <xsl:if test="position() > 1">,
              </xsl:if>
              <xsl:value-of select="@Name" />
            </xsl:for-each>
            
            <xsl:if test="Return">
              =
            </xsl:if>
            <b>
            <xsl:value-of select="$full_name" />
            </b>
              (
                <xsl:for-each select="Param">
                  <xsl:if test="position() > 1">,
                  </xsl:if>
                  <xsl:value-of select="@Name" />
                </xsl:for-each>
              )
          </td>
        </tr>
        <tr>
          <td colspan="2"><i><xsl:value-of select="Desc" /></i></td>
        </tr>
        
        <xsl:if test="Param">
        <tr>
          <td width="100" colspan="2"><b><u>Parameters:</u></b></td>
        </tr>
        </xsl:if>
        
        <xsl:apply-templates select="Param" /> 
        
        <xsl:if test="Return">
        <tr>
          <td width="100" colspan="2"><b><u>Return:</u></b></td>
        </tr>
        </xsl:if>
        
        <xsl:apply-templates select="Return" /> 
        
        <tr>
          <td width="100"><b><u>Example:</u></b></td>
          <td>
<pre>
<xsl:value-of select="Code" /> 
</pre>
          </td>
        </tr>
      </table>
    </p>
  </xsl:template>
  
  <xsl:template match="Prop">
    <xsl:variable name="full_name" select="concat(../@Name, '.', @Name)" />
    <p>
      <table border="1" cellpadding="5" cellspacing="0" width="100%">
        <tr>
          <td colspan="2"><big><b><a name="{$full_name}"><xsl:value-of select="$full_name" /></a></b> (property)</big></td>
        </tr>
        <tr>
          <td colspan="2"><i><xsl:value-of select="Desc" /></i></td>
        </tr>
        
        <tr>
          <td width="100"><b><u>Example:</u></b></td>
          <td>
<pre>
<xsl:value-of select="Code" /> 
</pre>
          </td>
        </tr>
      </table>
    </p>
  </xsl:template>
  
</xsl:stylesheet>
