/*
  Copyright (C) 2012  Torsten Brischalle

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/
*/

#include "crc.h"

//-----------------------------------------------------------------------------

#define CRC_CALC(NAME, NUM_TYPE)                                \
  void NAME(NUM_TYPE *crc_ptr, NUM_TYPE polynom, uint8_t data)  \
  {                                                             \
    unsigned int i;                                             \
    NUM_TYPE crc = *crc_ptr;                                    \
                                                                \
    for (i = 0; i < 8; i++)                                     \
    {                                                           \
      unsigned int b = (unsigned int)(crc & 1);                 \
                                                                \
      crc >>= 1;                                                \
                                                                \
      if (b != (data & 1))                                      \
        crc ^= polynom;                                         \
                                                                \
      data >>= 1;                                               \
    }                                                           \
                                                                \
    *crc_ptr = crc;                                             \
  }

//-----------------------------------------------------------------------------

CRC_CALC(crc8_calc,  uint8_t)
CRC_CALC(crc16_calc, uint16_t)
CRC_CALC(crc32_calc, uint32_t)
CRC_CALC(crc64_calc, uint64_t)

//-----------------------------------------------------------------------------

#define CRC_CALC_BUFFER(NAME, NUM_TYPE, CALC_FUNC)                                            \
  void NAME(NUM_TYPE *crc_ptr, NUM_TYPE polynom, const void *buffer, unsigned int buffer_sz)  \
  {                                                                                           \
    const uint8_t *buf = (const uint8_t *)buffer;                                             \
    unsigned int i;                                                                           \
                                                                                              \
    for (i = 0; i < buffer_sz; i++)                                                           \
      CALC_FUNC(crc_ptr, polynom, buf[i]);                                                    \
  }

//-----------------------------------------------------------------------------

CRC_CALC_BUFFER(crc8_calc_buffer,  uint8_t,  crc8_calc)
CRC_CALC_BUFFER(crc16_calc_buffer, uint16_t, crc16_calc)
CRC_CALC_BUFFER(crc32_calc_buffer, uint32_t, crc32_calc)
CRC_CALC_BUFFER(crc64_calc_buffer, uint64_t, crc64_calc)

//-----------------------------------------------------------------------------


