/*
  Copyright (C) 2012  Torsten Brischalle

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/
*/

#ifndef CRC_H
#define CRC_H

//-----------------------------------------------------------------------------

#include <stdint.h>

//-----------------------------------------------------------------------------

#define CRC8_POLYNOM_1_WIRE   0x8C

#define CRC16_POLYNOM_IBM     0xA001

#define CRC32_POLYNOM_IEEE    0xEDB88320UL

#define CRC64_POLYNOM_ECMA    0xC96C5795D7870F42ULL

//-----------------------------------------------------------------------------

#define CRC_CALC_PROTO(NAME, NUM_TYPE)          void NAME(NUM_TYPE *crc_ptr, NUM_TYPE polynom, uint8_t byte);

CRC_CALC_PROTO(crc8_calc,  uint8_t)
CRC_CALC_PROTO(crc16_calc, uint16_t)
CRC_CALC_PROTO(crc32_calc, uint32_t)
CRC_CALC_PROTO(crc64_calc, uint64_t)

//-----------------------------------------------------------------------------

#define CRC_CALC_BUFFER_PROTO(NAME, NUM_TYPE)   void NAME(NUM_TYPE *crc_ptr, NUM_TYPE polynom, const void *buffer, unsigned int buffer_sz);

CRC_CALC_BUFFER_PROTO(crc8_calc_buffer,  uint8_t)
CRC_CALC_BUFFER_PROTO(crc16_calc_buffer, uint16_t)
CRC_CALC_BUFFER_PROTO(crc32_calc_buffer, uint32_t)
CRC_CALC_BUFFER_PROTO(crc64_calc_buffer, uint64_t)

//-----------------------------------------------------------------------------

#endif

