/*
  Copyright (C) 2012  Torsten Brischalle

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/
*/

#include "fft.h"

#define _USE_MATH_DEFINES
#include <math.h>

#define FFT_SWAP(a,b) tempr = (a); (a)=(b); (b) = tempr

//-----------------------------------------------------------------------------

/*  Replaces data[0 .. 2 * nn-1] by its discrete Fourier transform, if isign is input as 1; or replaces
    data[0 .. 2 * nn-1] by nn times its inverse discrete Fourier transform, if isign is input as -1.
    data is a complex array of length nn or, equivalently, a real array of length 2*nn. nn MUST
    be an integer power of 2 (this is not checked for!).
*/

void fft(float data[], unsigned long nn, int isign)
{
  unsigned long n, mmax, m, j, istep, i;
  double        wtemp, wr, wpr, wpi, wi, theta;
  float         tempr, tempi;
  
  // the index shall start at 0
  data--;

  n = nn << 1;
  j = 1;
  for (i = 1; i < n; i += 2)
  {
    if (j > i)
    {
      FFT_SWAP(data[j], data[i]);
      FFT_SWAP(data[j + 1], data[i + 1]);
    }
    m = n >> 1;
    while ((m >= 2) && (j > m))
    {
      j -= m;
      m >>= 1;
    }
    j += m;
  }
  
  mmax = 2;
  while (n > mmax)
  {
    istep = mmax << 1;
    theta = isign * (2 * M_PI / mmax);
    wtemp = sin(0.5 * theta);
    wpr = -2 * wtemp * wtemp;
    wpi = sin(theta);
    wr = 1.0;
    wi = 0.0;
    for (m = 1; m < mmax; m += 2)
    {
      for (i = m; i <= n; i += istep)
      {
        j = i + mmax;
        tempr = (float)(wr * data[j] - wi * data[j + 1]);
        tempi = (float)(wr * data[j + 1] + wi * data[j]);
        data[j] = data[i] - tempr;
        data[j + 1] = data[i + 1] - tempi;
        data[i] += tempr;
        data[i + 1] += tempi;
      }
      wtemp = wr;
      wr = wr * wpr - wi * wpi + wr;
      wi = wi * wpr + wtemp * wpi + wi;
    }
    mmax = istep;
  }
}

//-----------------------------------------------------------------------------
