/*
  Copyright (C) 2012  Torsten Brischalle

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/
*/

#ifndef QGFXVIEWPIXMAP_H
#define QGFXVIEWPIXMAP_H

//-----------------------------------------------------------------------------

#include <QScrollArea>
#include <QPixmap>
#include <QPainter>
#include <QSharedPointer>
#include <QGraphicsView>
#include <QGraphicsScene>
#include <QGraphicsItem>

#include "gfxview.h"

//-----------------------------------------------------------------------------

class GfxItemPixmap : public QGraphicsItem
{
public:

  QPixmap m_pixmap;

  explicit GfxItemPixmap(QGraphicsItem *parent = 0);

  QRectF boundingRect() const;
  void paint(QPainter *painter, const QStyleOptionGraphicsItem *item, QWidget *widget);
};

//-----------------------------------------------------------------------------

class GfxViewPixmap : public GfxView
{
  Q_OBJECT
public:

  GfxItemPixmap   *m_gfxItemPixmap;

  explicit GfxViewPixmap(QWidget *parent = 0);

private:
  QPainter        m_painter;
  QPen            m_pen;
  QBrush          m_brush;
  QGraphicsScene  m_gfxScene;
  bool            m_preventGfxUpdate;
  bool            m_drawing;
  bool            m_fill;

  void BeginDraw(bool fill = false);
  void EndDraw();

signals:

public slots:
  void SlotGfxClear();
  void SlotGfxSetSize(int x, int y);
  void SlotGfxDrawPoint(int x, int y);
  void SlotGfxDrawLine(int x1, int y1, int x2, int y2);
  void SlotGfxDrawCircle(int x, int y, int r);
  void SlotGfxDrawEllipse(int x, int y, int rx, int ry);
  void SlotGfxDrawRect(int x, int y, int w, int h);
  void SlotGfxDrawPolygon(const QPolygon & polygon);
  void SlotGfxFillCircle(int x, int y, int r);
  void SlotGfxFillEllipse(int x, int y, int rx, int ry);
  void SlotGfxFillRect(int x, int y, int w, int h);
  void SlotGfxFillPolygon(const QPolygon & polygon);
  void SlotGfxDrawText(int x, int y, const QString & text);
  void SlotGfxSetColor(int red, int green, int blue);
  void SlotGfxSetColor(const QString & name);
  void SlotGfxSetFillColor(int red, int green, int blue);
  void SlotGfxSetFillColor(const QString & name);
  void SlotGfxBeginUpdate();
  void SlotGfxEndUpdate();
  void SlotGfxSaveImg(const QString & fileName);
};

//-----------------------------------------------------------------------------

#endif // QGFXVIEW_H
