/*
  Copyright (C) 2012  Torsten Brischalle

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/
*/

#include "register_lua_functions.h"

#include "lua_table_func.h"

#include "lua_func_sys.h"
#include "lua_func_bit.h"
#include "lua_func_dia.h"
#include "lua_func_math_ex.h"
#include "lua_func_string.h"
#include "lua_func_array.h"
#include "lua_func_crc.h"
#include "lua_func_tab.h"
#include "lua_func_gfx.h"
#include "lua_func_vecgfx.h"

//-----------------------------------------------------------------------------

const lua_table_function sys_func_list[] =
{
  { "delay", lua_func_delay },
  { 0, 0 }
};

const lua_table_property sys_prop_list[] =
{
  { "time_str", lua_prop_time_str_read, 0 },
  { "time", lua_prop_time_read, 0 },
  { 0, 0, 0 }
};

//-----------------------------------------------------------------------------

const lua_table_function bit_func_list[] =
{
  { "or", lua_func_bit_or },
  { "and", lua_func_bit_and },
  { "xor", lua_func_bit_xor },
  { "nand", lua_func_bit_nand },
  { "nor", lua_func_bit_nor },
  { "xnor", lua_func_bit_xnor },
  { "not", lua_func_bit_not },
  { "get", lua_func_bit_get },
  { "set", lua_func_bit_set },
  { "clear", lua_func_bit_clear },
  { "toggle", lua_func_bit_toggle },
  { "shr", lua_func_bit_shr },
  { "shl", lua_func_bit_shl },
  { "reverse", lua_func_bit_reverse },
  { 0, 0 }
};

const lua_table_property bit_prop_list[] =
{
  { 0, 0, 0 }
};

//-----------------------------------------------------------------------------

const lua_table_function dia_func_list[] =
{
  { "new", lua_func_dia_new },
  { "delete_all", lua_func_dia_delete_all },
  { 0, 0 }
};

const lua_table_property dia_prop_list[] =
{
  { 0, 0, 0 }
};

//-----------------------------------------------------------------------------

const lua_table_function tab_func_list[] =
{
  { "clear", lua_func_tab_clear },
  { "set_size", lua_func_tab_set_size },
  { "set_value", lua_func_tab_set_value },
  { "set_row_values", lua_func_tab_set_row_values },
  { "set_column_values", lua_func_tab_set_column_values },
  { "save_csv", lua_func_tab_save_csv },
  { 0, 0 }
};

const lua_table_property tab_prop_list[] =
{
  { 0, 0, 0 }
};

//-----------------------------------------------------------------------------

const lua_table_function gfx_func_list[] =
{
  { "set_size", lua_func_gfx_set_size },
  { "clear", lua_func_gfx_clear },
  { "draw_pnt", lua_func_gfx_draw_pnt },
  { "draw_line", lua_func_gfx_draw_line },
  { "draw_circle", lua_func_gfx_draw_circle },
  { "draw_ellipse", lua_func_gfx_draw_ellipse },
  { "draw_rect", lua_func_gfx_draw_rect },
  { "draw_polygon", lua_func_gfx_draw_polygon },
  { "fill_circle", lua_func_gfx_fill_circle },
  { "fill_ellipse", lua_func_gfx_fill_ellipse },
  { "fill_rect", lua_func_gfx_fill_rect },
  { "fill_polygon", lua_func_gfx_fill_polygon },
  { "draw_text", lua_func_gfx_draw_text },
  { "set_color", lua_func_gfx_set_color },
  { "set_fill_color", lua_func_gfx_set_fill_color },
  { "begin_update", lua_func_gfx_begin_update },
  { "end_update", lua_func_gfx_end_update },
  { "save_img", lua_func_gfx_save_img },
  { 0, 0 }
};

const lua_table_property gfx_prop_list[] =
{
  { 0, 0, 0 }
};

//-----------------------------------------------------------------------------

const lua_table_function vecgfx_func_list[] =
{
  { "clear", lua_func_vecgfx_clear },
  { "draw_line", lua_func_vecgfx_draw_line },
  { "draw_circle", lua_func_vecgfx_draw_circle },
  { "draw_ellipse", lua_func_vecgfx_draw_ellipse },
  { "draw_rect", lua_func_vecgfx_draw_rect },
  { "draw_polygon", lua_func_vecgfx_draw_polygon },
  { "fill_circle", lua_func_vecgfx_fill_circle },
  { "fill_ellipse", lua_func_vecgfx_fill_ellipse },
  { "fill_rect", lua_func_vecgfx_fill_rect },
  { "fill_polygon", lua_func_vecgfx_fill_polygon },
  { "draw_text", lua_func_vecgfx_draw_text },
  { "set_color", lua_func_vecgfx_set_color },
  { "set_fill_color", lua_func_vecgfx_set_fill_color },
  { "save_svg", lua_func_vecgfx_save_svg },
  { "save_img", lua_func_vecgfx_save_img },
  { 0, 0 }
};

const lua_table_property vecgfx_prop_list[] =
{
  { 0, 0, 0 }
};

//-----------------------------------------------------------------------------

const lua_table_function math_ex_func_list[] =
{
  { "calc_fft", lua_func_calc_fft },
  { "lin_reg", lua_func_lin_reg },
  { "median_filter", lua_func_median_filter },
  { "mean_filter", lua_func_mean_filter },
  { "mean", lua_func_mean },
  { 0, 0 }
};

const lua_table_property math_ex_prop_list[] =
{
  { 0, 0, 0 }
};

//-----------------------------------------------------------------------------

const lua_table_function array_func_list[] =
{
  { "print", lua_func_print_array },
  { "make", lua_func_make_array },
  { "reverse", lua_func_reverse_array },
  { "mul", lua_func_mul_array },
  { "div", lua_func_div_array },
  { "sub", lua_func_sub_array },
  { "add", lua_func_add_array },
  { "extract", lua_func_extract_array },
  { "max", lua_func_array_max },
  { "min", lua_func_array_min },
  { "to_string", lua_func_array_to_string },
  { "to_number", lua_func_array_to_number },
  { 0, 0 }
};

const lua_table_property array_prop_list[] =
{
  { 0, 0, 0 }
};

//-----------------------------------------------------------------------------

const lua_table_function crc_func_list[] =
{
  { "crc8", lua_func_crc8 },
  { "crc16", lua_func_crc16 },
  { "crc32", lua_func_crc32 },
  { 0, 0 }
};

const lua_table_property crc_prop_list[] =
{
  { "polynom8", lua_prop_crc8_polynom_read, lua_prop_crc8_polynom_write },
  { "polynom16", lua_prop_crc16_polynom_read, lua_prop_crc16_polynom_write },
  { "polynom32", lua_prop_crc32_polynom_read, lua_prop_crc32_polynom_write },
  { 0, 0, 0 }
};

//-----------------------------------------------------------------------------

const lua_table_function string_ex_func_list[] =
{
  { "split", lua_func_string_split },
  { 0, 0 }
};

const lua_table_property string_ex_prop_list[] =
{
  { 0, 0, 0 }
};

//-----------------------------------------------------------------------------

void register_lua_functions(lua_State *lua)
{
  lua_global_table_begin(lua, "sys");
  lua_global_table_end(lua, "sys", sys_func_list, sys_prop_list);

  lua_global_table_begin(lua, "bit");
  lua_global_table_end(lua, "bit", bit_func_list, bit_prop_list);
  
  lua_global_table_begin(lua, "dia");
  lua_global_table_end(lua, "dia", dia_func_list, dia_prop_list);
  
  lua_global_table_begin(lua, "tab");
  lua_global_table_end(lua, "tab", tab_func_list, tab_prop_list);

  lua_global_table_begin(lua, "gfx");
  lua_global_table_end(lua, "gfx", gfx_func_list, gfx_prop_list);

  lua_global_table_begin(lua, "vgfx");
  lua_global_table_end(lua, "vgfx", vecgfx_func_list, vecgfx_prop_list);

  lua_global_table_begin(lua, "math_ex");
  lua_global_table_end(lua, "math_ex", math_ex_func_list, math_ex_prop_list);
  
  lua_global_table_begin(lua, "array");
  lua_global_table_end(lua, "array", array_func_list, array_prop_list);
  
  lua_global_table_begin(lua, "crc");
  lua_global_table_end(lua, "crc", crc_func_list, crc_prop_list);
  
  lua_global_table_begin(lua, "string_ex");
  lua_global_table_end(lua, "string_ex", string_ex_func_list, string_ex_prop_list);
}

//-----------------------------------------------------------------------------

void register_int(lua_State *lua, const char *name, int value)
{
  lua_pushinteger(lua, value);
  lua_setfield(lua, -2, name);
}

//-----------------------------------------------------------------------------

void register_double(lua_State *lua, const char *name, double value)
{
  lua_pushnumber(lua, value);
  lua_setfield(lua, -2, name);
}

//-----------------------------------------------------------------------------

